/*! Tripetto Collector 0.4.23 - Copyright (C) 2018 Tripetto B.V. - All Rights Reserved */

/// <reference path="marblecore.functional.d.ts" />

declare module "@marblecore/hookup" {
    /** Explicit exports */
    export { Hooks, THooks, IHookPayload } from "@marblecore/hookup/lib/hooks";
    export { IHookBinding } from "@marblecore/hookup/lib/binding";
    /** Default export */
    export { Hooks as default } from "@marblecore/hookup/lib/hooks";
    /** Decorators */
    export { hookup } from "@marblecore/hookup/lib/decorator";
}

declare module "@marblecore/hookup/lib/hooks" {
    /** Declares the hooks type which is used to define the hook names. */
    export type THooks = string | void;
    /** Declares the hook types. */
    export type THookType = "synchronous" | "asynchronous" | "framed";
    /** Declares the hook context type. */
    export type THookContext = Function | {};
    /** Declares the hook function. */
    export type THookFunc<
        Names extends THooks,
        Hook extends Names,
        Payload extends IHookPayload<Hook> = IHookPayload<Hook>
    > = (pPayload: Payload) => void;
    /** Describes the hook payload interface. */
    export interface IHookPayload<Name extends THooks> {
        Hook?: {
            /** Contains the hook name. */
            Name: Name;
            /** Contains the hook type. */
            Type: THookType;
        };
    }
    /**
     * Implements the hooks class.
     * @param Hooks Specifies the available hooks (specify using string literals).
     */
    export class Hooks<Names extends THooks> {
        /** Retrieves if the hooks buffer is enabled. */
        readonly IsHookBufferEnabled: boolean;
        /**
         * Register a hook.
         * @param sHook Specifies the hook name.
         * @param sType Specifies the hook type.
         * @param fnHook Specifies the hook function.
         * @param pContext Specifies the hook context.
         * @return Returns `this` to allow chaining.
         */
        Hook<Hook extends Names, Payload extends IHookPayload<Hook>>(
            sHook: Hook,
            sType: THookType,
            fnHook: THookFunc<Names, Hook, Payload>,
            pContext?: THookContext
        ): this;
        /**
         * Register a grouped hook.
         * @param sHook Specifies the hook name.
         * @param sType Specifies the hook type.
         * @param fnHook Specifies the hook function.
         * @param pContext Specifies the hook context.
         * @return Returns `this` to allow chaining.
         */
        GroupedHook<Hook extends Names, Payload extends IHookPayload<Hook>>(
            sHook: Hook,
            sType: THookType,
            fnHook: THookFunc<Names, Hook, Payload>,
            pContext?: THookContext
        ): this;
        /**
         * Emits a hook.
         * @param sHook Specifies the hook name.
         * @param pPayload Specifies the payload (optional).
         * @return Returns `true` if the hook is emitted.
         */
        EmitHook<Hook extends Names, Payload extends IHookPayload<Hook>>(
            sHook: Hook,
            pPayload: Payload,
            sMode?: "fifo" | "lifo"
        ): boolean;
        /** Enables the hooks buffer. */
        BufferHooks(): void;
        /** Flushes the hooks buffer. */
        FlushHooks(): void;
        /**
         * Removes a hook.
         * @param sHook Specifies the hook name.
         * @param pContextOrFunction Specifies the hook context or function to detach.
         * @return Returns `true` if the hook was found and removed.
         */
        Unhook<Hook extends Names>(
            sHook: Hook,
            pContextOrFunction?: THookFunc<Names, Hook> | THookContext
        ): boolean;
        /**
         * Unhooks all hooks from the specifies context.
         * @param pContext Specifies the context.
         * @return Returns `true` if one or more hooks were found and removed for
         * the specifies context.
         */
        UnhookContext(pContext: THookContext): boolean;
        /** Destroys the hooks (any pending hook emits are flushed synchronous). */
        Destroy(): void;
    }
}

declare module "@marblecore/hookup/lib/binding" {
    /** Dependencies */
    import { IHookPayload, THooks } from "@marblecore/hookup/lib/hooks";
    /** Declares the hook binding interface. */
    export interface IHookBinding<
        Hook extends THooks,
        T extends IHookPayload<Hook>
    > {
        Hook: Hook;
        Payload: T;
    }
}

declare module "@marblecore/hookup/lib/decorator" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { Hooks, THooks } from "@marblecore/hookup/lib/hooks";
    /**
     * Binds a hook to a property.
     * @param sHook Specifies the hook to attach.
     */
    export function hookup<T extends Hooks<F.TAny>, Hook extends THooks>(
        sHook: Hook
    ): (
        pTarget: T,
        sProperty: string,
        pDescriptor?: PropertyDescriptor
    ) => void;
}
