/*! Tripetto Collector 0.4.23 - Copyright (C) 2018 Tripetto B.V. - All Rights Reserved */

/// <reference path="marblecore.hookup.d.ts" />
/// <reference path="marblecore.functional.d.ts" />

declare module "@marblecore/matrix" {
    /** Explicit exports */
    export { ICursor, ISafeCursor } from "@marblecore/matrix/lib/cursor";
    export { Matrix } from "@marblecore/matrix/lib/matrix";
    export { Item } from "@marblecore/matrix/lib/item";
    export { StrictMatrix } from "@marblecore/matrix/lib/strict/matrix";
    export { StrictItem } from "@marblecore/matrix/lib/strict/item";
    export {
        IStrictItemConstructor
    } from "@marblecore/matrix/lib/strict/constructor";
    export { Model } from "@marblecore/matrix/lib/model/model";
    export {
        ModelItem,
        IDataChangeEvent
    } from "@marblecore/matrix/lib/model/item";
    /** Hooks */
    export {
        IDimensionChangeEvent,
        IItemPushEvent,
        IItemChangeEvent,
        IItemPopEvent,
        ICursorChangeEvent
    } from "@marblecore/matrix/lib/events";
    /** Default export */
    export { Matrix as default } from "@marblecore/matrix/lib/matrix";
    /** Enumerations */
    export { Orientations } from "@marblecore/matrix/enum/orientations";
    export { Reasons } from "@marblecore/matrix/enum/reasons";
    export { Types } from "@marblecore/matrix/enum/types";
}

declare module "@marblecore/matrix/lib/cursor" {
    export interface ICursor {
        /** Specifies the current column of the cursor. */
        Column?: number;
        /** Specifies the current row of the cursor. */
        Row?: number;
        /** Specifies the current index of the cursor. */
        Index?: number;
    }
    export interface ISafeCursor {
        /** Specifies the current column of the cursor. */
        Column: number;
        /** Specifies the current row of the cursor. */
        Row: number;
        /** Specifies the current index of the cursor. */
        Index: number;
    }
}

declare module "@marblecore/matrix/lib/matrix" {
    import { Hooks, THooks } from "@marblecore/hookup";
    import { Item } from "@marblecore/matrix/lib/item";
    import { ICursor, ISafeCursor } from "@marblecore/matrix/lib/cursor";
    import {
        ICursorChangeEvent,
        IDimensionChangeEvent,
        TMatrixHooks
    } from "@marblecore/matrix/lib/events";
    import { Orientations } from "@marblecore/matrix/enum/orientations";
    import { Types } from "@marblecore/matrix/enum/types";
    /**
     * Implements the convolution matrix.
     * @param T Specifies the item type.
     * @param H Specifies custom hooks for the matrix (optional).
     */
    export abstract class Matrix<
        T extends Item<THooks>,
        H extends THooks = void
    > extends Hooks<TMatrixHooks | H> {
        /**
         * Creates a new matrix.
         * @param nType Specifies the matrix direction.
         * @param nOrientation Specifies the matrix orientation.
         * @param nDimensions Specifies the number of columns for vertical matrices or the number of rows for horizontal matrices.
         */
        constructor(
            nType?: Types,
            nOrientation?: Orientations,
            nDimensions?: number
        );
        /** Specifies the unique id of the matrix. */
        readonly Id: number;
        /** Sets the current matrix type. */
        Type: Types;
        /** Sets the current matrix orientation. */
        Orientation: Orientations;
        /** Sets the number of columns in the matrix. */
        Columns: number;
        /** Sets the number of rows in the matrix. */
        Rows: number;
        /** Retrieves the total number of items in the matrix. */
        readonly Count: number;
        /** Retrieves all the items. */
        readonly All: T[];
        /** Retrieves the item at the current cursor position. */
        readonly CurrentItem: T | undefined;
        /** Retrieves the first item in the matrix. */
        readonly FirstItem: T | undefined;
        /** Retrieves the last item in the matrix. */
        readonly LastItem: T | undefined;
        /** Retrieves the last item in the first row of the matrix. */
        readonly LastItemInFirstRow: T | undefined;
        /** Retrieves the last item in the first column of the matrix. */
        readonly LastItemInFirstColumn: T | undefined;
        /** Sets the current cursor. */
        Cursor: ICursor | ISafeCursor;
        /** Retrieves the current (safe) cursor. */
        readonly SafeCursor: ISafeCursor;
        /** Verifies if the cursor is at the first column. */
        readonly IsCursorAtFirstColumn: boolean;
        /** Verifies if the cursor is at the last column. */
        readonly IsCursorAtLastColumn: boolean;
        /** Verifies if the column cursor is past the last column. */
        readonly IsCursorAtEndColumn: boolean;
        /** Verifies if the row cursor is at the first row. */
        readonly IsCursorAtFirstRow: boolean;
        /** Verifies if the row cursor is at the last row. */
        readonly IsCursorAtLastRow: boolean;
        /** Verifies if the row cursor is past the last row. */
        readonly IsCursorAtEndRow: boolean;
        /** Verifies if the cursor is at the first item of the matrix. */
        readonly IsCursorAtFirst: boolean;
        /** Verifies if the cursor is at the last item of the matrix. */
        readonly IsCursorAtLast: boolean;
        /** Verifies if the cursor is past the end of the matrix. */
        readonly IsCursorAtEnd: boolean;
        /** Sets the silent mode. */
        IsSilent: boolean | (() => boolean);
        /**
         * Retrieves a validated cursor from the supplied cursor.
         * @param pCursor Specifies the source cursor.
         * @return Returns the validated cursor.
         */
        protected GetSafeCursor(pCursor: ICursor): ISafeCursor;
        /**
         * Fired when the dimensions of the matrix are changed.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected OnDimensionChange(pEvent: IDimensionChangeEvent<this>): void;
        /**
         * Fired when the cursor position of the matrix is changed.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected OnCursorChange(pEvent: ICursorChangeEvent<this>): void;
        /**
         * Retrieves the index for the given column and row.
         * @param nColumn Specifies the column.
         * @param nRow Specifies the row.
         * @return Returns the item index or `-1` if the index is invalid.
         */
        IndexFromColumnRow(nColumn: number, nRow: number): number;
        /**
         * Retrieves a cursor for the specified index.
         * @param nIndex Specifies the index.
         * @return Returns a cursor.
         */
        CursorFromIndex(nIndex: number | undefined): ISafeCursor;
        /**
         * Pushes an item to the matrix.
         * @param pItem Specifies the item to put in the matrix.
         * @param pOptions Specifies one or more of the following options:
         * - Index: Specifies the desired index for the item (if `undefined` the
         * supplied index in the item will be used or the item will be pushed to the
         * end of the matrix if the index is invalid);
         * - Recalculate: Specifies if the matrix needs to be recalculated;
         * - Emit: Specifies what to emit.
         * @return Returns a reference to the item.
         */
        PushToMatrix(
            pItem: T,
            pOptions?: {
                Index: number | undefined;
                Recalculate: boolean;
                Emit: "nothing" | "push" | "change";
            }
        ): T;
        /**
         * Moves an item to the specified cursor.
         * @param pItem Specifies the item to move.
         * @param pCursor Specifies the cursor.
         * @return Returns `true` if the move succeeded.
         */
        MoveTo(pItem: T, pCursor: ICursor): boolean;
        /**
         * Moves an item to the specified index position.
         * @param pItem Specifies the item to move.
         * @param nIndex Specifies the index position.
         * @return Returns `true` if the move succeeded.
         */
        MoveToIndex(pItem: T, nIndex: number): boolean;
        /**
         * Moves an item to the specified cursor.
         * @param pItem Specifies the item to move.
         * @param nColumn Specifies the column.
         * @param nRow Specifies the row.
         * @return Returns `true` if the move succeeded.
         */
        MoveToPosition(pItem: T, nColumn: number, nRow: number): boolean;
        /**
         * Moves an item to the specified item.
         * @param pItem Specifies the item to swap.
         * @param pTo Specifies the item to move to.
         * @param bAbove Specifies if the move position is above or below the specified item.
         * @return Returns `true` if the move succeeded.
         */
        MoveToItem(pItem: T, pTo: T, bAbove: boolean): boolean;
        /**
         * Moves an item up or down.
         * @param pItem Specifies the item to move.
         * @param bUp Specifies if the move direction is up (true) or down (false).
         * @return Returns `true` if the move succeeded.
         */
        MoveUpOrDown(pItem: T, bUp: boolean): boolean;
        /**
         * Swaps two items.
         * @param pA Item A.
         * @param pB Item B.
         * @return Returns `true` if the swap succeeded.
         */
        Swap(pA: T, pB: T): boolean;
        /**
         * Removes an item from the matrix (or the last item if no item is specified).
         * @param pItem Specifies the item to remove.
         * @param bEmit Specifies if the events needs to be emitted.
         * @return Returns the popped item.
         */
        PopFromMatrix(pItem?: T, bEmit?: boolean): T | undefined;
        /**
         * Sorts the matrix using the supplied compare functions.
         * @param fnCompare Specifies the compare function. The compare function should return a number according to the following rules:
         * - A is less than B return -1;
         * - A is greater than B return 1;
         * - A is equal to B return 0.
         *
         * To compare numbers you can simply return `nA - nB`. If no compare function is specified, the items are sorted according to the
         * original push order.
         * @param fnCompare.pA Compare item A.
         * @param fnCompare.pB Compare item B.
         * @return Returns `true` if the sort invoked an item position change.
         */
        Sort(fnCompare?: (pA: T, pB: T) => number): boolean;
        /**
         * Removes all matrix items.
         * @param sMode Specifies the clear mode:
         * - `dirty`: Just removes all the items from the matrix without emitting events;
         * - `fast`: Removes the items and invokes the `OnPop` method for each item;
         * - `thorough`: Removes the items and invokes the `OnPop`method and hooks for each item.
         */
        Clear(sMode?: "dirty" | "fast" | "thorough"): void;
        /** Invalidates the matrix and forces the `OnDimensionChange` event. */
        InvalidateMatrix(): void;
        /** Recalculates the matrix. */
        RecalculateMatrix(): void;
        /**
         * Iterates through each item.
         * @param fnCallee Specifies the function to be invoked for each item. The item value will be exposed to the function as the first
         * argument of the argument list. Additional arguments can be specified and will be pushed to the function. If the function returns
         * `true` the iteration is stopped.
         * @param nFrom Specifies the index to start the iteration from. A negative index indicates an offset from the end of the matrix.
         * @param nTo Specifies the index to stop the iteration. A negative index indicates an offset from the end of the matrix.
         * @return Returns `true` if the iteration is stopped (break).
         */
        Each(
            fnCallee: (pItem: T) => boolean | void,
            nFrom?: number,
            nTo?: number
        ): boolean;
        /**
         * Iterates through each item starting with the last item.
         * @param fnCallee Specifies the function to be invoked for each item. The item value will be exposed to the function as the first
         * argument of the argument list. Additional arguments can be specified and will be pushed to the function. If the function
         * returns `true` the iteration is stopped.
         * @param nFrom Specifies the index to start the iteration from. A negative index indicates an offset from the end of the matrix.
         * @param nTo Specifies the index to stop the iteration. A negative index indicates an offset from the end of the matrix.
         * @return Returns `true` if the iteration is be stopped (break).
         */
        ReverseEach(
            fnCallee: (pItem: T) => boolean | void,
            nFrom?: number,
            nTo?: number
        ): boolean;
        /**
         * Retrieves the item at the specified index.
         * @param nIndex Specifies the index.
         * @return Returns the item or `undefined` if the item is not found.
         */
        ItemAtIndex(nIndex: number): T | undefined;
        /**
         * Retrieves the item at the specified position.
         * @param nColumn Specifies the column.
         * @param nRow Specifies the row.
         * @return Returns the item or `undefined` if the item is not found.
         */
        ItemAtPosition(nColumn: number, nRow: number): T | undefined;
        /**
         * Retrieves the item at the specified cursor.
         * @param pCursor Specifies the cursor.
         * @return Returns the item or `undefined` if the item is not found.
         */
        ItemAtCursor(pCursor: ICursor): T | undefined;
        /**
         * Retrieves the item with the specified id.
         * @param nId Specifies the id of the item.
         * @return Returns the item or `undefined` if the item is not found.
         */
        ItemFromId(nId: number): T | undefined;
        /**
         * Retrieves the last item in the specified row of the matrix.
         * @param nRow Specifies the row.
         * @return Returns a reference to the last item or `undefined` if the item is not found.
         */
        LastItemInRow(nRow: number): T | undefined;
        /**
         * Retrieves the last item in the specified column of the matrix.
         * @param nColumn Specifies the column.
         * @return Returns a reference to the last item or `undefined` if the item is not found.
         */
        LastItemInColumn(nColumn: number): T | undefined;
        /**
         * Moves the cursor to the first column.
         * @param bMoveToFirstRow Specifies if the cursor needs to be moved to the first row.
         * @return Returns the new cursor position.
         */
        MoveCursorToFirstColumn(bMoveToFirstRow?: boolean): ISafeCursor;
        /**
         * Moves the cursor to the next column.
         * @param bMoveToFirstRow Specifies if the cursor needs to be moved to the first row.
         * @return Returns the new cursor position.
         */
        MoveCursorToSuccessorColumn(bMoveToFirstRow?: boolean): ISafeCursor;
        /**
         * Moves the cursor to the previous column.
         * @param bMoveToFirstRow Specifies if the cursor needs to be moved to the first row.
         * @return Returns the new cursor position.
         */
        MoveCursorToPredecessorColumn(bMoveToFirstRow?: boolean): ISafeCursor;
        /**
         * Moves the cursor to the last column.
         * @param bMoveToFirstRow Specifies if the cursor needs to be moved to the first row.
         * @return Returns the new cursor position.
         */
        MoveCursorToLastColumn(bMoveToFirstRow?: boolean): ISafeCursor;
        /**
         * Moves the cursor to the first row.
         * @param bMoveToFirstColumn Specifies if the cursor needs to be moved to the first column.
         * @return Returns the new cursor position.
         */
        MoveCursorToFirstRow(bMoveToFirstColumn?: boolean): ISafeCursor;
        /**
         * Moves the cursor to the next row.
         * @param bMoveToFirstColumn Specifies if the cursor needs to be moved to the first column.
         * @return Returns the new cursor position.
         */
        MoveCursorToSuccessorRow(bMoveToFirstColumn?: boolean): ISafeCursor;
        /**
         * Moves the cursor to the previous row.
         * @param bMoveToFirstColumn Specifies if the cursor needs to be moved to the first column.
         * @return Returns the new cursor position.
         */
        MoveCursorToPredecessorRow(bMoveToFirstColumn?: boolean): ISafeCursor;
        /**
         * Moves the row cursor to the last row.
         * @param bMoveToFirstColumn Specifies if column cursor needs to be moved to the first column.
         * @return Returns the new cursor position.
         */
        MoveCursorToLastRow(bMoveToFirstColumn?: boolean): ISafeCursor;
        /**
         * Moves the cursor to the first item.
         * @return Returns the new cursor position.
         */
        MoveCursorToFirst(): ISafeCursor;
        /**
         * Moves the cursor to the next item.
         * @return Returns the new cursor position.
         */
        MoveCursorToSuccessor(): ISafeCursor;
        /**
         * Moves the cursor to the previous item.
         * @return Returns the new cursor position.
         */
        MoveCursorToPredecessor(): ISafeCursor;
        /**
         * Moves the cursor to the last item.
         * @return Returns the new cursor position.
         */
        MoveCursorToLast(): ISafeCursor;
        /**
         * Moves the cursor past the last item.
         * @return Returns the new cursor position.
         */
        MoveCursorToEnd(): ISafeCursor;
    }
}

declare module "@marblecore/matrix/lib/item" {
    /** Dependencies */
    import * as F from "@marblecore/functional";
    import { Hooks, IHookBinding, THooks } from "@marblecore/hookup";
    import { Matrix } from "@marblecore/matrix/lib/matrix";
    import { ICursor } from "@marblecore/matrix/lib/cursor";
    import {
        IItemChangeEvent,
        IItemPopEvent,
        IItemPushEvent,
        TItemHooks
    } from "@marblecore/matrix/lib/events";
    import { Stack } from "@marblecore/matrix/lib/stack";
    import { Reasons } from "@marblecore/matrix/enum/reasons";
    /**
     * Implements the matrix item.
     * @param H Specifies custom hooks for the matrix (optional).
     */
    export abstract class Item<H extends THooks = void> extends Hooks<
        TItemHooks | H
    > {
        /** Contains the id enumerator. */
        static readonly Enumerator: F.Enumerator;
        /**
         * Pushes an item to the supplied matrix.
         * @param pItem Reference to the item.
         * @param pMatrix Reference to the matrix to push the item to. If omitted
         * the current matrix of the item will be used.
         * @return Returns a reference to the item.
         */
        static PushToMatrix<T extends Item<THooks>>(
            pItem: T,
            pMatrix?: Matrix<T, THooks>
        ): T;
        /**
         * Creates a new item.
         * @param pMatrix Optional reference to the matrix.
         */
        constructor(pMatrix?: Matrix<Item<H>, THooks>);
        /** Retrieves a reference to the underlying matrix. */
        readonly Parent: Matrix<Item<H>, THooks> | undefined;
        /** Specifies the unique id of the item. The id is unique within the current session. */
        readonly Id: number;
        /** Overrides the automatic calculated hash value for the item. */
        Hash: string;
        /** Sets the index of the item (first item is zero). */
        Index: number;
        /** Returns the column index of the item in the matrix (first column is zero). */
        readonly ColumnIndex: number;
        /** Returns the row index of the item in the matrix (first row is zero). */
        readonly RowIndex: number;
        /** Retrieves if the item is popped (not in the matrix). */
        readonly IsPopped: boolean;
        /** Specifies if the item is the first item in the column or row. */
        readonly IsFirst: boolean;
        /** Specifies if the item is the last item in the column or row. */
        readonly IsLast: boolean;
        /** Specifies if the item is the first item in the column. */
        readonly IsFirstColumn: boolean;
        /** Specifies if the item is the last item in the column. */
        readonly IsLastColumn: boolean;
        /** Specifies if the item is the first item in the row. */
        readonly IsFirstRow: boolean;
        /** Specifies if the item is the last item in the row. */
        readonly IsLastRow: boolean;
        /** Specifies if the item is the first item in the matrix. */
        readonly IsFirstOfMatrix: boolean;
        /** Specifies if the item is the last item in the matrix. */
        readonly IsLastOfMatrix: boolean;
        /** Retrieves the previous sibling of the item. */
        readonly PredecessorSibling: Item<H> | undefined;
        /** Retrieves the next sibling of the item. */
        readonly SuccessorSibling: Item<H> | undefined;
        /** Retrieves the previous item in the same column. */
        readonly PredecessorInColumn: Item<H> | undefined;
        /** Retrieves the next item in the same column. */
        readonly SuccessorInColumn: Item<H> | undefined;
        /** Retrieves the previous item in the same row. */
        readonly PredecessorInRow: Item<H> | undefined;
        /** Retrieves the next item in the same row. */
        readonly SuccessorInRow: Item<H> | undefined;
        /**
         * Retrieves a reference to the underlying matrix.
         * @return Returns a reference to the matrix.
         */
        protected Matrix(): Matrix<Item<H>, THooks> | undefined;
        /**
         * Fired when the item is pushed.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected OnPush(pEvent: IItemPushEvent<Item<H>>): void;
        /**
         * Fired when the item is changed.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected OnChange(pEvent: IItemChangeEvent<Item<H>>): void;
        /**
         * Fired when the item is popped.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected OnPop(pEvent: IItemPopEvent<Item<H>>): void;
        /**
         * Dispatches a matrix event.
         * @param pStack Reference to the stack.
         * @param pEvent Specifies the event to dispatch.
         */
        Dispatch(
            pStack: Stack,
            pEvent:
                | IHookBinding<"OnItemChange", IItemChangeEvent<Item<H>>>
                | IHookBinding<"OnItemPop", IItemPopEvent<Item<H>>>
                | "Pop"
                | "Detach"
        ): void;
        /**
         * Sets the index of an item.
         * @param pStack Reference to the stack.
         * @param pMatrix Reference to the matrix.
         * @param nIndex Specifies the new index of the item.
         * @param sEmit Specifies if events needs to be emitted.
         * @param nReason Specifies the event reason.
         * @return Returns `true` if a change occured.
         */
        Reindex(
            pStack: Stack,
            pMatrix: Matrix<Item<H>, THooks>,
            nIndex: number,
            sEmit: "nothing" | "auto" | "change" | "push",
            nReason: Reasons
        ): boolean;
        /**
         * Moves the item to the specified cursor.
         * @param pCursor Specifies the cursor.
         * @return Returns `true` if the move succeeded.
         */
        MoveTo(pCursor: ICursor): boolean;
        /**
         * Moves the item to the specified cursor.
         * @param nIndex Specifies the index position.
         * @return Returns `true` if the move succeeded.
         */
        MoveToIndex(nIndex: number): boolean;
        /**
         * Moves the item to the specified cursor.
         * @param nColumn Specifies the column.
         * @param nRow Specifies the row.
         * @return Returns `true` if the move succeeded.
         */
        MoveToPosition(nColumn: number, nRow: number): boolean;
        /**
         * Moves the item to the specified item.
         * @param pTo Specifies the item to move to.
         * @param bAbove Specifies if the move position is above or below the
         * specified item.
         * @return Returns `true` if the move succeeded.
         */
        MoveToItem(pTo: Item<H>, bAbove: boolean): boolean;
        /**
         * Moves the item up or down.
         * @param bUp Specifies if the move direction is up (true) or down (false).
         * @return Returns `true` if the move succeeded.
         */
        MoveUpOrDown(bUp: boolean): boolean;
        /**
         * Swaps an item with another item.
         * @param pWith Specifies the item to swap with.
         * @return Returns `true` if the swap succeeded.
         */
        Swap(pWith: Item<H>): boolean;
        /**
         * Removes the item from the matrix.
         * @return Returns `true` if the function succeeded.
         */
        Pop(): boolean;
        /**
         * Removes the item from the matrix without emitting events.
         * @return Returns `true` if the function succeeded.
         */
        PopSilent(): boolean;
        /**
         * Restores a popped item.
         * @param pMatrix Specifies a matrix to use.
         * @param nIndex Specifies the restore position.
         * @return Returns a reference to the item.
         */
        Unpop(pMatrix?: Matrix<Item<H>, THooks>, nIndex?: number): boolean;
        /**
         * Restores a popped item.
         * @param pMatrix Specifies a matrix to use.
         * @param nIndex Specifies the restore position.
         * @return Returns a reference to the item.
         */
        UnpopSilent(
            pMatrix?: Matrix<Item<H>, THooks>,
            nIndex?: number
        ): boolean;
    }
}

declare module "@marblecore/matrix/lib/strict/matrix" {
    import { THooks } from "@marblecore/hookup";
    import { Matrix } from "@marblecore/matrix/lib/matrix";
    import { ICursor } from "@marblecore/matrix/lib/cursor";
    import { Orientations } from "@marblecore/matrix/enum/orientations";
    import { Types } from "@marblecore/matrix/enum/types";
    import { StrictItem } from "@marblecore/matrix/lib/strict/item";
    import { IStrictItemConstructor } from "@marblecore/matrix/lib/strict/constructor";
    /**
     * Implements the strict convolution matrix. In a strict matrix an item is
     * always bound to a parent matrix and items can be created by the matrix.
     * @param P Specifies the matrix type.
     * @param T Specifies the item type.
     * @param H Specifies custom hooks for the matrix (optional).
     * @param D Specifies a data type for the item (optional).
     */
    export abstract class StrictMatrix<
        P extends StrictMatrix<P, T, H, D>,
        T extends StrictItem<P, THooks, D>,
        H extends THooks = void,
        D = undefined
    > extends Matrix<T, H> {
        /**
         * Creates a new matrix.
         * @param pItem Specifies the item constructor.
         * @param nType Specifies the matrix direction.
         * @param nOrientation Specifies the matrix orientation.
         * @param nDimensions Specifies the number of columns for vertical matrices or the number of rows for horizontal matrices.
         */
        constructor(
            pItemConstructor: IStrictItemConstructor<P, T, D>,
            nType?: Types,
            nOrientation?: Orientations,
            nDimensions?: number
        );
        /** Retrieves a reference to the item constructor. */
        readonly ItemConstructor: IStrictItemConstructor<P, T, D>;
        /**
         * Appends a new item to the end of the matrix.
         * @param pData Data which is passed to the item constructor.
         * @return Returns a reference to the item.
         */
        protected AppendToMatrix(pData: D): T;
        /**
         * Inserts a new item at the beginning of the matrix.
         * @param pData Data which is passed to the item constructor.
         * @return Returns a reference to the item.
         */
        protected InsertToMatrix(pData: D): T;
        /**
         * Inserts a new item at the specified cursor or at the beginning of the matrix.
         * @param pData Data which is passed to the item constructor.
         * @param pCursor Specifies the cursor.
         * @return Returns a reference to the item.
         */
        protected InsertToMatrixAtCursor(pData: D, pCursor: ICursor): T;
        /**
         * Inserts a new item at the specified index.
         * @param pData Data which is passed to the item constructor.
         * @param nIndex Specifies the index position.
         * @return Returns a reference to the item.
         */
        protected InsertToMatrixAtIndex(pData: D, nIndex: number): T;
        /**
         * Inserts a new item at the specified index.
         * @param pData Data which is passed to the item constructor.
         * @param nColumn Specifies the column.
         * @param nRow Specifies the row.
         * @return Returns a reference to the item.
         */
        protected InsertToMatrixAtPosition(
            pData: D,
            nColumn: number,
            nRow: number
        ): T;
        /**
         * Invoked when a new item instance is requested.
         * @param pData Specifies the data.
         * @return Returns the new item instance.
         * @event
         */
        OnItemInstance(pData: D): T;
    }
}

declare module "@marblecore/matrix/lib/strict/item" {
    /** Dependencies */
    import { THooks } from "@marblecore/hookup";
    import { Item } from "@marblecore/matrix/lib/item";
    import { StrictMatrix } from "@marblecore/matrix/lib/strict/matrix";
    /**
     * Implements the strict matrix item. In a strict matrix an item is always
     * bound to a parent matrix and items can be created by the matrix.
     * @param P Specifies the matrix type.
     * @param H Specifies custom hooks for the item (optional).
     * @param D Specifies a data type for the item (optional).
     */
    export abstract class StrictItem<
        P extends StrictMatrix<P, StrictItem<P, H, D>, THooks, D>,
        H extends THooks = void,
        D = undefined
    > extends Item<H> {
        /**
         * Creates a new strict item.
         * @param pMatrix Reference to the matrix.
         */
        constructor(pMatrix: P);
        /** Retrieves a reference to the parent matrix. */
        readonly Parent: P;
        /**
         * Retrieves a reference to the underlying matrix.
         * @return Returns a reference to the matrix.
         */
        protected Matrix(): P;
        /**
         * Inserts a new item before this item.
         * @param pData Data which is passed to the item constructor.
         * @return Returns a reference to the item.
         */
        protected InsertBefore(pData: D): StrictItem<P, H, D>;
        /**
         * Inserts a new item after this item.
         * @param pData Data which is passed to the item constructor.
         * @return Returns a reference to the item.
         */
        protected InsertAfter(pData: D): StrictItem<P, H, D>;
    }
}

declare module "@marblecore/matrix/lib/strict/constructor" {
    /** Dependencies */
    import { THooks } from "@marblecore/hookup";
    import { StrictItem } from "@marblecore/matrix/lib/strict/item";
    import { StrictMatrix } from "@marblecore/matrix/lib/strict/matrix";
    /**
     * Describes the item constructor.
     * @param P Specifies the matrix type.
     * @param T Specifies the item type.
     * @param D Specifies a data type for the item (optional).
     */
    export interface IStrictItemConstructor<
        P extends StrictMatrix<P, T, THooks, D>,
        T extends StrictItem<P, THooks, D>,
        D
    > {
        new (pMatrix: P, pData: D): T;
    }
}

declare module "@marblecore/matrix/lib/model/model" {
    import { StrictMatrix } from "@marblecore/matrix/lib/strict/matrix";
    import { ICursor } from "@marblecore/matrix/lib/cursor";
    import { Orientations } from "@marblecore/matrix/enum/orientations";
    import { Types } from "@marblecore/matrix/enum/types";
    import { ModelItem } from "@marblecore/matrix/lib/model/item";
    /**
     * Creates a matrix model of the specified type.
     * @param T Specifies the type for the model.
     */
    export class Model<T> extends StrictMatrix<
        Model<T>,
        ModelItem<T>,
        "OnDataChange",
        T
    > {
        /**
         * Creates a new matrix.
         * @param nType Specifies the matrix direction.
         * @param nOrientation Specifies the matrix orientation.
         * @param nDimensions Specifies the number of columns for vertical matrices
         * or the number of rows for horizontal matrices.
         * @param pItems Specifies an array with initial data items.
         */
        constructor(
            nType?: Types,
            nOrientation?: Orientations,
            nDimensions?: number,
            pItems?: T[]
        );
        /**
         * Appends a new item to the end of the model.
         * @param pData Specifies the data for the item.
         * @return Returns the new item.
         */
        Append(pData: T): ModelItem<T>;
        /**
         * Inserts a new item at the beginning of the model.
         * @param pData Specifies the data for the item.
         * @return Returns the new item.
         */
        Insert(pData: T): ModelItem<T>;
        /**
         * Inserts a new item at the specified cursor.
         * @param pData Specifies the data for the item.
         * @param pCursor Specifies the cursor.
         * @return Returns the new item.
         */
        InsertAtCursor(pData: T, pCursor: ICursor): ModelItem<T>;
        /**
         * Inserts a new item at the specified index.
         * @param pData Specifies the data for the item.
         * @param nIndex Specifies the index position.
         * @return Returns the new item.
         */
        InsertAtIndex(pData: T, nIndex: number): ModelItem<T>;
        /**
         * Inserts a new item at the specified index.
         * @param pData Specifies the data for the item.
         * @param nColumn Specifies the column.
         * @param nRow Specifies the row.
         * @return Returns the new item.
         */
        InsertAtPosition(pData: T, nColumn: number, nRow: number): ModelItem<T>;
    }
}

declare module "@marblecore/matrix/lib/model/item" {
    /** Dependencies */
    import { IHookPayload } from "@marblecore/hookup";
    import { StrictItem } from "@marblecore/matrix/lib/strict/item";
    import { Model } from "@marblecore/matrix/lib/model/model";
    /** Defines the `OnDataChange` event. */
    export interface IDataChangeEvent<T> extends IHookPayload<"OnDataChange"> {
        /** Reference to the model. */
        readonly Model: Model<T>;
        /** Reference to the item. */
        readonly Item: ModelItem<T>;
        /** Reference to the data. */
        readonly Data: T;
    }
    /**
     * Creates a new matrix model item.
     * @param T Specifies the type for the item.
     */
    export class ModelItem<T> extends StrictItem<Model<T>, "OnDataChange", T> {
        /**
         * Creates a new item.
         * @param pData Specifies the data for the item.
         */
        constructor(pMatrix: Model<T>, pData: T);
        /** Sets the attached data. */
        Data: T;
        /**
         * Inserts a new item before this item.
         * @param pData Specifies the data for the item.
         * @return Returns the new item.
         */
        InsertBefore(pData: T): ModelItem<T>;
        /**
         * Inserts a new item after this item.
         * @param pData Specifies the data for the item.
         * @return Returns the new item.
         */
        InsertAfter(pData: T): ModelItem<T>;
    }
}

declare module "@marblecore/matrix/lib/events" {
    /** Dependencies */
    import { IHookPayload, THooks } from "@marblecore/hookup";
    import { Matrix } from "@marblecore/matrix/lib/matrix";
    import { Item } from "@marblecore/matrix/lib/item";
    import { ICursor } from "@marblecore/matrix/lib/cursor";
    import { Reasons } from "@marblecore/matrix/enum/reasons";
    /** Defines the matrix hooks. */
    export type TMatrixHooks =
        | "OnDimensionChange"
        | "OnCursorChange"
        | "OnItemPush"
        | "OnItemChange"
        | "OnItemPop";
    /** Defines the item hooks. */
    export type TItemHooks = "OnItemPush" | "OnItemChange" | "OnItemPop";
    /** Defines the `DimensionChange` event. */
    export interface IDimensionChangeEvent<
        T extends Matrix<Item<THooks>, THooks>
    > extends IHookPayload<"OnDimensionChange"> {
        /** Reference to the matrix. */
        readonly Matrix: T;
        /** Specifies the matrix volume difference since the last event. */
        readonly Difference: number;
        /** Specifies the number of columns. */
        readonly Columns: number;
        /** Specifies the number of rows. */
        readonly Rows: number;
        /** Specifies the reason for the change. */
        readonly Reason: Reasons;
    }
    /** Defines the `Item` event which is the base event for `IItemPushEvent` and `IItemChangeEvent`. */
    export interface IItemEvent<H extends THooks, T extends Item<THooks>>
        extends IHookPayload<H> {
        /** Reference to the current matrix. */
        readonly Matrix: Matrix<T, THooks>;
        /** Reference to the previous matrix. */
        readonly MatrixPrevious: Matrix<T, THooks> | undefined;
        /** Reference to the item. */
        readonly Item: T;
        /** Specifies the reason for the push. */
        readonly Reason: Reasons;
    }
    /** Defines the `ItemPush` event. */
    export interface IItemPushEvent<T extends Item<THooks>>
        extends IItemEvent<"OnItemPush", T> {}
    /** Defines the `ItemChange` event. */
    export interface IItemChangeEvent<T extends Item<THooks>>
        extends IItemEvent<"OnItemChange", T> {}
    /** Defines the `ItemPop` event. */
    export interface IItemPopEvent<T extends Item<THooks>>
        extends IHookPayload<"OnItemPop"> {
        /** Reference to the matrix. */
        readonly Matrix: Matrix<T, THooks>;
        /** Reference to the item. */
        readonly Item: T;
    }
    /** Defines the `CursorChange` event. */
    export interface ICursorChangeEvent<T extends Matrix<Item<THooks>, THooks>>
        extends IHookPayload<"OnCursorChange"> {
        /** Reference to the matrix. */
        readonly Matrix: T;
        /** Contains the cursor position. */
        readonly Cursor: ICursor;
    }
}

declare module "@marblecore/matrix/enum/orientations" {
    export const enum Orientations {
        /** Items a vertical oriented (columns are filled out). */
        Vertical = 0,
        /** Items are horizontal oriented (rows are filled out). */
        Horizontal = 1
    }
}

declare module "@marblecore/matrix/enum/reasons" {
    export const enum Reasons {
        /** A new item is pushed to the matrix. */
        Push = 1,
        /** An item is popped from the matrix. */
        Pop = 2,
        /** An item is moved to another position in the matrix. */
        Move = 4,
        /** An item is swapped. */
        Swap = 8,
        /** The matrix is sorted. */
        Sort = 16,
        /** A matrix recalculation occurred. */
        Recalculation = 32,
        /** An item has become the last item or the predecessor of the last item. */
        Last = 64,
        /** An item has become the first item or the successor of the first item. */
        First = 128
    }
}

declare module "@marblecore/matrix/enum/types" {
    export const enum Types {
        /** A vertical matrix expands vertical (fixed number of columns). */
        Vertical = 0,
        /** A horizontal matrix expands horizontal (fixed number of rows). */
        Horizontal = 1,
        /** A square matrix expands horizontal and vertical in a perfect square. */
        Square = 2,
        /** A horizontal square matrix expands first horizontal and then vertical. */
        SquareHorizontal = 3,
        /** A vertical square matrix expands first vertical and then horizontal. */
        SquareVertical = 4
    }
}

declare module "@marblecore/matrix/lib/stack" {
    export class Stack {
        /**
         * Pushes a call to the stack.
         * @param fnCall Specifies the call.
         */
        Push(fnCall: () => void): void;
        /** Flushes the stack. */
        Flush(): void;
    }
}
