/*! Tripetto Collector 0.4.23 - Copyright (C) 2018 Tripetto B.V. - All Rights Reserved */

declare module "@marblecore/metrics" {
    /** Explicit exports */
    export { IPoint } from "@marblecore/metrics/lib/point";
    export { IRectangleProperties } from "@marblecore/metrics/lib/properties";
    export { Rectangle } from "@marblecore/metrics/lib/rectangle";
    export { Rectangles } from "@marblecore/metrics/lib/rectangles";
    export { Coordinate } from "@marblecore/metrics/lib/coordinate";
    export { Dimensions } from "@marblecore/metrics/lib/dimensions";
    export { Spacing } from "@marblecore/metrics/lib/spacing";
}

declare module "@marblecore/metrics/lib/point" {
    export interface IPoint {
        /** Specifies the horizontal coordinate. */
        readonly X: number;
        /** Specifies the vertical coordinate. */
        readonly Y: number;
    }
}

declare module "@marblecore/metrics/lib/properties" {
    export interface IRectangleProperties<T extends string> {
        /** Specifies the rectangle name. */
        Name: T;
        /** Specifies the initial width of the rectangle. */
        readonly Width?: number;
        /** Specifies the initial height of the rectangle. */
        readonly Height?: number;
        /** Specifies the minimal width of the rectangle. */
        readonly MinWidth?: number;
        /** Specifies the minimal height of the rectangle. */
        readonly MinHeight?: number;
        /** Specifies the maximal width of the rectangle. */
        readonly MaxWidth?: number;
        /** Specifies the minimal width of the rectangle. */
        readonly MaxHeight?: number;
        /** Specifies the left spacing of the rectangle. */
        readonly SpacingLeft?: number;
        /** Specifies the top spacing of the rectangle. */
        readonly SpacingTop?: number;
        /** Specifies the right spacing of the rectangle. */
        readonly SpacingRight?: number;
        /** Specifies the bottom spacing of the rectangle. */
        readonly SpacingBottom?: number;
        /** Specifies the left hint correction. */
        readonly HintCorrectionLeft?: number;
        /** Specifies the top hint correction. */
        readonly HintCorrectionTop?: number;
        /** Specifies the right hint correction. */
        readonly HintCorrectionRight?: number;
        /** Specifies the bottom hint correction. */
        readonly HintCorrectionBottom?: number;
        /** Specifies the horizontal mean correction. */
        readonly MeanCorrectionHorizontal?: number;
        /** Specifies the vertical mean correction. */
        readonly MeanCorrectionVertical?: number;
        /** Specifies the names of the rectangles which should align to this rectangle. */
        readonly Align?: T[];
    }
}

declare module "@marblecore/metrics/lib/rectangle" {
    import { Coordinate } from "@marblecore/metrics/lib/coordinate";
    import { Dimensions } from "@marblecore/metrics/lib/dimensions";
    export class Rectangle {
        /**
         * Constructs a new rectangle.
         * @param pProperties Specifies the rectangle properties.
         */
        constructor(pProperties?: {
            /** Specifies the left position. */
            Left?: number;
            /** Specifies the top position. */
            Top?: number;
            /** Specifies the right position. */
            Right?: number;
            /** Specifies the bottom position. */
            Bottom?: number;
            /** Specifies the width of the rectangle. If the width is supplied the specified right position is ignored. */
            Width?: number;
            /** Specifies the height of the rectangle. If the height is supplied the specified bottom position is ignored. */
            Height?: number;
            /** Specifies the minimum width. */
            MinWidth?: number;
            /** Specifies the minimum height. */
            MinHeight?: number;
            /** Specifies the maximum width. */
            MaxWidth?: number;
            /** Specifies the maximum height. */
            MaxHeight?: number;
        });
        /** Retrieves the dimensions. */
        readonly Dimensions: Dimensions;
        /** Retrieves the coordinates. */
        readonly Coordinate: Coordinate;
        /** Sets the left coordinate of the rectangle. */
        Left: number;
        /** Sets the top coordinate of the rectangle. */
        Top: number;
        /** Sets the right coordinate of the rectangle. */
        Right: number;
        /** Sets the bottom coordinate of the rectangle. */
        Bottom: number;
        /** Sets the width of the rectangle. */
        Width: number;
        /** Sets the height of the rectangle. */
        Height: number;
        /** Retrieves the rounded width of the rectangle.  */
        readonly RoundedWidth: number;
        /** Retrieves the rounded height of the rectangle.  */
        readonly RoundedHeight: number;
        /** Retrieves the unranged width of the rectangle.  */
        readonly W: number;
        /** Retrieves the unranged height of the rectangle.  */
        readonly H: number;
        /** Retrieves an immutable copy of the rectangle. */
        readonly Copy: Rectangle;
        /**
         * Verifies if the supplied point is within the rectangle.
         * @param nX Specifies the horizontal position.
         * @param nY Specifies the vertical position.
         * @return Returns `true` if the point is within the rectangle.
         */
        IsIn(nX?: number, nY?: number): boolean;
    }
}

declare module "@marblecore/metrics/lib/rectangles" {
    import { Coordinate } from "@marblecore/metrics/lib/coordinate";
    import { Dimensions } from "@marblecore/metrics/lib/dimensions";
    import { Rectangle } from "@marblecore/metrics/lib/rectangle";
    import { IRectangleProperties } from "@marblecore/metrics/lib/properties";
    /**
     * Implements a rectangle manager which can be used to manage rectangle relationships. The relation between those rectangles is described
     * by specifying characteristics of the rectangles, such as padding, minimal and maximum dimensions, etc. When the dimensions of a
     * rectangle change, the resulting dimensions of the surrounding rectangles are calculated.
     * @param T Specifies the rectangle names.
     */
    export class Rectangles<T extends string> {
        /**
         * Constructs a new rectangle repository.
         * @param pRectangles Specifies an array with rectangles.
         */
        constructor(pRectangles?: IRectangleProperties<T>[]);
        /** Sets the left coordinate of the comprehensive rectangle. */
        Left: number;
        /** Sets the top coordinate of the comprehensive rectangle. */
        Top: number;
        /** Retrieves the right coordinate of the comprehensive rectangle. */
        readonly Right: number;
        /** Retrieves the bottom coordinate of the comprehensive rectangle. */
        readonly Bottom: number;
        /** Retrieves the width of the comprehensive rectangle. */
        readonly Width: number;
        /** Retrieves the height of the comprehensive rectangle. */
        readonly Height: number;
        /** Returns the width and height of the comprehensive rectangle. */
        readonly Dimensions: Dimensions;
        /**
         * Retrieves the inner left coordinate of the comprehensive rectangle. This
         * is the rectangle which includes all rectangles without spacing.
         */
        readonly InnerLeft: number;
        /**
         * Retrieves the inner top coordinate of the comprehensive rectangle. This
         * is the rectangle which includes all rectangles without spacing.
         */
        readonly InnerTop: number;
        /**
         * Retrieves the inner right coordinate of the comprehensive rectangle. This
         * is the rectangle which includes all rectangles without spacing.
         */
        readonly InnerRight: number;
        /**
         * Retrieves the inner bottom coordinate of the comprehensive rectangle.
         * This is the rectangle which includes all rectangles without spacing.
         */
        readonly InnerBottom: number;
        /**
         * Retrieves the inner width of the comprehensive rectangle. This is the
         * rectangle which includes all rectangles without spacing.
         */
        readonly InnerWidth: number;
        /**
         * Retrieves the inner height of the comprehensive rectangle. This is the
         * rectangle which includes all rectangles without spacing.
         */
        readonly InnerHeight: number;
        /** Sets the shift mode. */
        IsShifted: boolean;
        /** Sets the left move. */
        ShiftLeft: number;
        /** Sets the top shift. */
        ShiftTop: number;
        /** Retrieves the right position of the shifted element. */
        readonly ShiftRight: number;
        /** Retrieves the bottom position of the shifted element. */
        readonly ShiftBottom: number;
        /** Retrieves the last horizontal position shift. */
        readonly ShiftHorizontal: number;
        /** Retrieves the last vertical position shift. */
        readonly ShiftVertical: number;
        /** Retrieves the horizontal shift mean. */
        readonly ShiftMeanHorizontal: number;
        /** Retrieves the vertical shift mean. */
        readonly ShiftMeanVertical: number;
        /**
         * Iterates through all rectangles.
         * @param fnRectangle Specifies the function which is called for each rectangle.
         * @param fnRectangle.pRectangle Reference to the rectangle metrics.
         * @param fnRectangle.sRectangle Contains the rectangle name.
         * @param nZoom Specifies an optional zoom factor.
         */
        Each(
            fnRectangle: (pRectangle: Rectangle, sRectangle: T) => void,
            nZoom?: number
        ): void;
        /**
         * Sets or retrieves the minimum width of the specified rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nMinWidth Specifies the minimum width.
         * @return Returns the minimal width.
         */
        MinWidth(sRectangle: T, nMinWidth?: number): number;
        /**
         * Sets or retrieves the minimum height of the specified rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nMinHeight Specifies the minimum height.
         * @return Returns the minimal height.
         */
        MinHeight(sRectangle: T, nMinHeight?: number): number;
        /**
         * Sets or retrieves the maximum width of the specified rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nMaxWidth Specifies the maximum width.
         * @return Returns the maximum width.
         */
        MaxWidth(sRectangle: T, nMaxWidth?: number): number;
        /**
         * Sets or retrieves the maximum height of the specified rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nMaxHeight Specifies the maximum height.
         * @return Returns the maximum height.
         */
        MaxHeight(sRectangle: T, nMaxHeight?: number): number;
        /**
         * Retrieves the horizontal center of the specified rectangle or the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the mean of the comprehensive rectangle will be calculated.
         * @return Returns the horizontal mean.
         */
        MeanHorizontal(sRectangle?: T): number;
        /**
         * Retrieves the vertical center of the specified rectangle or the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the mean of the comprehensive rectangle will be calculated.
         * @return Returns the vertical mean.
         */
        MeanVertical(sRectangle?: T): number;
        /**
         * Retrieves the rectangle for the specified rectangle or the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nZoom Specifies an optional zoom factor.
         * @return Returns the rectangle.
         */
        Rectangle(sRectangle?: T, nZoom?: number): Rectangle;
        /**
         * Retrieves the applied rectangle for the specified rectangle. This is the rectangle which is currently stored in the buffer.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nZoom Specifies an optional zoom factor.
         * @return Returns the rectangle.
         */
        RectangleApplied(sRectangle?: T, nZoom?: number): Rectangle;
        /**
         * Sets or retrieves the width of a rectangle or retrieves the width of the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nWidth Specifies the width.
         * @return Returns the width.
         */
        RectangleWidth(sRectangle?: T, nWidth?: number): number;
        /**
         * Verifies if the specified rectangle is aligned with another rectangle.
         * @param sRectangleA Specifies the first rectangle.
         * @param sRectangleB Specifies the second rectangle.
         * @return Returns `true` if the rectangle is aligned.
         */
        IsRectangleAligned(sRectangleA: T, sRectangleB: T): boolean;
        /**
         * Sets or retrieves the height of a rectangle or retrieves the width of the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nWidth Specifies the width.
         * @return Returns the width.
         */
        RectangleHeight(sRectangle?: T, nHeight?: number): number;
        /**
         * Sets or retrieves the left spacing of a rectangle or retrieves the left comprehensive rectangle spacing.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nSpacingLeft Specifies the left spacing.
         * @return Returns the left spacing.
         */
        SpacingLeft(sRectangle?: T, nSpacingLeft?: number): number;
        /**
         * Sets or retrieves the top spacing of a rectangle or retrieves the top comprehensive rectangle spacing.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nSpacingTop Specifies the top spacing.
         * @return Returns the top spacing.
         */
        SpacingTop(sRectangle?: T, nSpacingTop?: number): number;
        /**
         * Sets or retrieves the right spacing of a rectangle or retrieves the right comprehensive rectangle spacing.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nSpacingLeft Specifies the right spacing.
         * @return Returns the right spacing.
         */
        SpacingRight(sRectangle?: T, nSpacingRight?: number): number;
        /**
         * Sets or retrieves the bottom spacing of a rectangle or retrieves the bottom comprehensive rectangle spacing.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @param nSpacingBottom Specifies the bottom spacing.
         * @return Returns the bottom spacing.
         */
        SpacingBottom(sRectangle?: T, nSpacingBottom?: number): number;
        /**
         * Retrieves the horizontal spacing of a rectangle or the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @return Returns the horizontal spacing.
         */
        SpacingHorizontal(sRectangle?: T): number;
        /**
         * Retrieves the vertical spacing of a rectangle or the comprehensive rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @return Returns the vertical spacing.
         */
        SpacingVertical(sRectangle?: T): number;
        /**
         * Sets or retrieves the left hint correction of a rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nHintLeft Specifies the left hint correction.
         * @return Returns the left hint correction.
         */
        HintCorrectionLeft(sRectangle: T, nHintLeft?: number): number;
        /**
         * Sets or retrieves the top hint correction of a rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nHintTop Specifies the top hint correction.
         * @return Returns the top hint correction.
         */
        HintCorrectionTop(sRectangle: T, nHintTop?: number): number;
        /**
         * Sets or retrieves the right hint correction of a rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nHintRight Specifies the right hint correction.
         * @return Returns the right hint correction.
         */
        HintCorrectionRight(sRectangle: T, nHintRight?: number): number;
        /**
         * Sets or retrieves the bottom hint correction of a rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nHintBottom Specifies the bottom hint correction.
         * @return Returns the bottom hint correction.
         */
        HintCorrectionBottom(sRectangle: T, nHintBottom?: number): number;
        /**
         * Retrieves the shifted rectangle.
         * @param sRectangle Specifies the name of the rectangle. If omitted the rectangle of the comprehensive rectangle will be calculated.
         * @return Returns the rectangle.
         */
        ShiftRectangle(sRectangle?: T): Rectangle;
        /**
         * Positions the rectangles.
         * @return Returns `true` if the position is changed.
         */
        Position(): boolean;
        /**
         * Resizes the rectangles.
         * @return Returns which properties are changed (`none`, `width`, `height` or `both`).
         */
        Resize(): "none" | "width" | "height" | "both";
        /**
         * Verifies if the supplied coordinate is within one or more rectangles.
         * @param nX Specifies the left position.
         * @param nY Specifies the top position.
         * @param bSpacing Specifies if the spacing needs to be applied.
         * @return Returns `true` if the coordinate is within one or more rectangles.
         */
        IsCoordinateIn(
            nX: number | undefined,
            nY: number | undefined,
            bSpacing?: boolean
        ): boolean;
        /**
         * Verifies if the supplied coordinate is within the comprehensive rectangle.
         * @param nX Specifies the left position.
         * @param nY Specifies the top position.
         * @param bInner Specifies if the inner rectangle needs to be verified.
         * @param pCorrection Specifies correction coordinates.
         * @return Returns `true` if the coordinate is within the comprehensive rectangle.
         */
        IsCoordinateInRectangles(
            nX: number | undefined,
            nY: number | undefined,
            bInner?: boolean,
            pCorrection?: Coordinate
        ): boolean;
        /**
         * Verifies if the supplied coordinate is within the specified rectangle.
         * @param sRectangle Specifies the name of the rectangle.
         * @param nX Specifies the left position.
         * @param nY Specifies the top position.
         * @param bSpacing Specifies if spacing needs to be included.
         * @return Returns `true` if the coordinate is within the specified rectangle.
         */
        IsCoordinateInRectangle(
            sRectangle: T,
            nX: number | undefined,
            nY: number | undefined,
            bSpacing?: boolean
        ): boolean;
    }
}

declare module "@marblecore/metrics/lib/coordinate" {
    export class Coordinate {
        /** Specifies the left coordinate. */
        Left: number;
        /** Specifies the top coordinate. */
        Top: number;
        /** Specifies the right coordinate. */
        Right: number;
        /** Specifies the bottom coordinate. */
        Bottom: number;
        /**
         * Constructs a new coordinate.
         * @param nLeft Specifies the left coordinate.
         * @param nTop Specifies the top coordinate.
         * @param nRight Specifies the right coordinate.
         * @param nBottom Specifies the bottom coordinate.
         */
        constructor(
            nLeft?: number,
            nTop?: number,
            nRight?: number,
            nBottom?: number
        );
    }
}

declare module "@marblecore/metrics/lib/dimensions" {
    export class Dimensions {
        /** Retrieves the `none` constant for the width and height. */
        static readonly NONE: number;
        /** Contains the current width. */
        m_nWidth: number;
        /** Contains the current height. */
        m_nHeight: number;
        /** Contains the minimum width. */
        m_nMinWidth: number;
        /** Contains the minimum height. */
        m_nMinHeight: number;
        /** Contains the maximum width. */
        m_nMaxWidth: number;
        /** Contains the maximum height. */
        m_nMaxHeight: number;
        /**
         * Constructs a new dimension object.
         * @param nWidth Specifies the width.
         * @param nHeight Specifies the height.
         * @param nMinWidth Specifies the minimum width.
         * @param nMinHeight Specifies the minimum height.
         * @param nMaxWidth Specifies the maximum width.
         * @param nMaxHeight Specifies the minimum height.
         */
        constructor(
            nWidth?: number,
            nHeight?: number,
            nMinWidth?: number,
            nMinHeight?: number,
            nMaxWidth?: number,
            nMaxHeight?: number
        );
        /** Sets the minimum width. */
        MinWidth: number;
        /** Sets the minimum height. */
        MinHeight: number;
        /** Sets the maximum width. */
        MaxWidth: number;
        /** Sets the maximum height. */
        MaxHeight: number;
        /** Sets the current width. */
        Width: number;
        /** Sets the current height. */
        Height: number;
        /** Retrieves if the width is specified. */
        readonly IsWidthSpecified: boolean;
        /** Retrieves if the height is specified. */
        readonly IsHeightSpecified: boolean;
        /** Retrieves the unranged width. */
        readonly W: number;
        /** Retrieves the unranged height. */
        readonly H: number;
    }
}

declare module "@marblecore/metrics/lib/spacing" {
    /** Dependencies */
    import { Coordinate } from "@marblecore/metrics/lib/coordinate";
    export class Spacing extends Coordinate {
        /**
         * Constructs a new spacing container.
         * @param nLeft Specifies the left spacing.
         * @param nTop Specifies the top spacing.
         * @param nRight Specifies the right spacing.
         * @param nBottom Specifies the bottom spacing.
         */
        constructor(
            nLeft?: number,
            nTop?: number,
            nRight?: number,
            nBottom?: number
        );
        /** Retrieves the total horizontal spacing. */
        readonly Horizontal: number;
        /** Retrieves the total vertical spacing. */
        readonly Vertical: number;
    }
}
